"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTemporaryFilePath = exports.moveFileIntoKacheryStorage = exports.concatenateFilesIntoTemporaryFile = void 0;
const crypto_1 = __importDefault(require("crypto"));
const fs_1 = __importDefault(require("fs"));
const util_1 = require("../../../common/util");
const core_1 = require("../../../interfaces/core");
const concatenateFilesIntoTemporaryFile = async (paths, storageDir) => {
    return new Promise((resolve, reject) => {
        const tmpPath = exports.createTemporaryFilePath({ storageDir, prefix: 'kachery-p2p-concat-' });
        const writeStream = fs_1.default.createWriteStream(tmpPath);
        const sha = crypto_1.default.createHash('sha1');
        let done = false;
        let ii = 0;
        const _handleNextFile = () => {
            if (done)
                return;
            if (ii >= paths.length) {
                writeStream.end(() => {
                    if (done)
                        return;
                    done = true;
                    const sha1 = sha.digest('hex');
                    resolve({ sha1, path: core_1.localFilePath(tmpPath) });
                });
                return;
            }
            const readStream = fs_1.default.createReadStream(paths[ii].toString());
            readStream.on('error', (err) => {
                if (done)
                    return;
                done = true;
                reject(err);
            });
            readStream.on('data', data => {
                if (done)
                    return;
                sha.update(data);
                writeStream.write(data);
            });
            readStream.on('end', () => {
                ii++;
                _handleNextFile();
            });
        };
        _handleNextFile();
    });
};
exports.concatenateFilesIntoTemporaryFile = concatenateFilesIntoTemporaryFile;
const moveFileIntoKacheryStorage = (storageDir, args) => {
    const s = args.sha1;
    const destParentPath = `${storageDir}/sha1/${s[0]}${s[1]}/${s[2]}${s[3]}/${s[4]}${s[5]}`;
    const destPath = `${destParentPath}/${s}`;
    if (fs_1.default.existsSync(destPath)) {
        fs_1.default.unlinkSync(args.path.toString());
        return destPath;
    }
    fs_1.default.mkdirSync(destParentPath, { recursive: true });
    fs_1.default.renameSync(args.path.toString(), destPath);
    return destPath;
};
exports.moveFileIntoKacheryStorage = moveFileIntoKacheryStorage;
const _getTemporaryDirectory = (storageDir) => {
    const ret = storageDir + '/tmp';
    mkdirIfNeeded(core_1.localFilePath(ret));
    return ret;
};
const mkdirIfNeeded = (path) => {
    if (!fs_1.default.existsSync(path.toString())) {
        try {
            fs_1.default.mkdirSync(path.toString());
        }
        catch (err) {
            if (!fs_1.default.existsSync(path.toString())) {
                fs_1.default.mkdirSync(path.toString());
            }
        }
    }
};
const createTemporaryFilePath = (args) => {
    const dirPath = _getTemporaryDirectory(args.storageDir);
    return `${dirPath}/${args.prefix}-${util_1.randomAlphaString(10)}`;
};
exports.createTemporaryFilePath = createTemporaryFilePath;
//# sourceMappingURL=concatenateFiles.js.map