"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _liveFeedInfos;
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../common/constants");
const GarbageMap_1 = __importDefault(require("../common/GarbageMap"));
const util_1 = require("../common/util");
const core_1 = require("../interfaces/core");
class RemoteFeedManager {
    // #remoteSubfeedSubscriptions = new GarbageMap<string, RemoteSubfeedSubscription>(null)
    // Manages interactions with feeds on remote nodes within the p2p network
    constructor(node, outgoingSubfeedSubscriptionManager) {
        this.node = node;
        this.outgoingSubfeedSubscriptionManager = outgoingSubfeedSubscriptionManager;
        _liveFeedInfos.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(5 * 60 * 1000))
        // #remoteSubfeedSubscriptions = new GarbageMap<string, RemoteSubfeedSubscription>(null)
        // Manages interactions with feeds on remote nodes within the p2p network
        ); // Information about the live feeds (cached in memory)
    }
    async subscribeToRemoteSubfeed(feedId, subfeedHash) {
        // todo: find the node ID and channel of the remote subfeed
        let info = __classPrivateFieldGet(this, _liveFeedInfos).get(feedId);
        if (!info) {
            const findLiveFeedResult = await this.node.findLiveFeed({ feedId, timeoutMsec: constants_1.TIMEOUTS.defaultRequest });
            if (!findLiveFeedResult)
                throw Error('Unable to find live feed (subscribeToRemoteSubfeed).');
            info = findLiveFeedResult;
            __classPrivateFieldGet(this, _liveFeedInfos).set(feedId, info);
        }
        const remoteNodeId = info.nodeId;
        // CHAIN:get_remote_messages:step(5)
        await this.outgoingSubfeedSubscriptionManager.createOrRenewOutgoingSubscription(remoteNodeId, feedId, subfeedHash);
    }
    async submitMessage(args) {
        const { feedId, subfeedHash, message, timeoutMsec } = args;
        // Submit messages to a subfeed on a remote node
        // This requires write permissions
        // Search and find the info for the feed (channel and nodeId)
        let waitMsec = 2000;
        let liveFeedInfo;
        while (true) {
            try {
                liveFeedInfo = await this.findLiveFeedInfo({ feedId, timeoutMsec });
                break;
            }
            catch (err) {
                if (waitMsec >= 2000) {
                    // wait and try again
                    await util_1.sleepMsec(core_1.scaledDurationMsec(2000));
                    waitMsec -= 2000;
                }
                else {
                    break;
                }
            }
        }
        // If we did not find it, then throw exception
        if (!liveFeedInfo) {
            throw Error(`Cannot find live feed: ${feedId}`);
        }
        // Now that we know the channel and nodeId, we can submit the messages via the swarm
        await this.node.submitMessageToRemoteLiveFeed({
            nodeId: liveFeedInfo.nodeId,
            feedId,
            subfeedHash,
            message,
            timeoutMsec
        });
    }
    async findLiveFeedInfo({ feedId, timeoutMsec }) {
        // Find the channel and nodeId for a feed that is owned by a remote node on the p2p network
        // If not found, throws an error.
        // First check if we have the information in the memory cache
        const cachedInfo = __classPrivateFieldGet(this, _liveFeedInfos).get(feedId);
        if (cachedInfo) {
            // check whether the node is still online
            if (this.node.remoteNodeManager().remoteNodeIsOnline(cachedInfo.nodeId)) {
                // if so, return it
                return cachedInfo;
            }
            else {
                // if not, then set the cached info to null
                __classPrivateFieldGet(this, _liveFeedInfos).delete(feedId);
            }
        }
        const x = await this.node.findLiveFeed({ feedId, timeoutMsec });
        if (x === null) {
            throw Error('Unable to find live feed.');
        }
        // Store in memory cache
        __classPrivateFieldGet(this, _liveFeedInfos).set(feedId, x);
        return x;
    }
}
_liveFeedInfos = new WeakMap();
exports.default = RemoteFeedManager;
//# sourceMappingURL=RemoteFeedManager.js.map